@@ -23,10 +23,9 @@ from scipy import sparse
 import requests  ## for downloading the KG
 
 ## modeling
-from sklearn.linear_model import SGDClassifier
-from sklearn.linear_model import SGDRegressor
-from sklearn.model_selection import GridSearchCV
-from sklearn.model_selection import ShuffleSplit
+from sklearn.linear_model import SGDClassifier, SGDRegressor
+from sklearn.model_selection import GridSearchCV, ShuffleSplit
+from sklearn.neighbors import KNeighborsClassifier, KNeighborsRegressor
 
 ## monitoring
 import tqdm
@@ -677,7 +676,30 @@ class GAlearner:
 
         if self.learner_hyperparameters is None:
 
-            if final_run:
+
+            ## this is for screening purposes.
+            if self.learner_preset == "default":
+                parameters = {
+                    "loss": ["hinge", "log"],
+                    "penalty": ["elasticnet"],
+                    "alpha": [0.01, 0.001, 0.0001],
+                    "l1_ratio": [0, 0.1, 0.5, 0.9]
+                }
+
+            elif self.learner_preset == "mini-l1":
+                parameters = {"loss": ["log"], "penalty": ["l1"]}
+
+            elif self.learner_preset == "mini-l2":
+                parameters = {"loss": ["log"], "penalty": ["l2"]}
+
+            elif self.learner_preset == "knn":
+                parameters = {"n_neighbors":list(range(1, 64, 1)),
+                              "weights":['uniform','distance'],
+                              "metric":["euclidean",
+                                        "manhattan",
+                                        "minkowski"]}
+
+            if final_run and self.learner_preset != "knn":
                 
                 ## we can afford this final round to be more rigorous.
                 parameters = {
@@ -685,24 +707,7 @@ class GAlearner:
                     "penalty": ["elasticnet"],
                     "alpha": [0.01, 0.001, 0.0001, 0.0005],
                     "l1_ratio": [0, 0.05, 0.25, 0.3, 0.6, 0.8, 0.95, 1]
-                }
-
-            else:
-                
-                ## this is for screening purposes.
-                if self.learner_preset == "default":
-                    parameters = {
-                        "loss": ["hinge", "log"],
-                        "penalty": ["elasticnet"],
-                        "alpha": [0.01, 0.001, 0.0001],
-                        "l1_ratio": [0, 0.1, 0.5, 0.9]
-                    }
-
-                elif self.learner_preset == "mini-l1":
-                    parameters = {"loss": ["log"], "penalty": ["l1"]}
-
-                elif self.learner_preset == "mini-l2":
-                    parameters = {"loss": ["log"], "penalty": ["l2"]}
+                }                
 
         else:
 
@@ -711,11 +716,18 @@ class GAlearner:
         if self.learner is None:
             
             if self.task == "classification":
-                svc = SGDClassifier(max_iter = 1000000)
+
+                if self.learner_preset == "knn":
+                    svc = KNeighborsClassifier()
+                else:
+                    svc = SGDClassifier(max_iter = 1000000)
                 
             else:
-                svc = SGDRegressor(max_iter = 1000000)
-                parameters['loss'] = ['squared_loss']
+                if self.learner_preset == "knn":
+                    svc = KNeighborsClassifier()
+                else:
+                    svc = SGDRegressor(max_iter = 1000000)
+                    parameters['loss'] = ['squared_loss']
 
         else:
             svc = self.learner
@@ -1320,10 +1332,14 @@ class GAlearner:
         dfx.columns = ['Importance', 'Feature subspace']
         
         ## store global top features
-        feature_ranking = self.global_feature_map  ## all features
+        try:
+            feature_ranking = self.global_feature_map  ## all features
+            
+        except:
+            feature_ranking = pd.DataFrame({k:1 for k in self.feature_names})
+            
         return feature_ranking, dfx
 
-
     def get_topic_explanation(self):
 
         """
@@ -1598,28 +1614,34 @@ class GAlearner:
                             f"Evaluation of individual {top_individual} did not produce a viable learner. Increase time!"
                         )
 
-                coefficients = learner.best_estimator_.coef_
+                try:
+                    coefficients = learner.best_estimator_.coef_
 
-                ## coefficients are given for each class. We take maximum one (abs val)
-                coefficients = np.asarray(np.abs(np.max(coefficients,
-                                                        axis = 0))).reshape(-1)
+                    ## coefficients are given for each class. We take maximum one (abs val)
+                    coefficients = np.asarray(np.abs(np.max(coefficients,
+                                                            axis = 0))).reshape(-1)
 
-                if self.verbose:
-                    logging.info("Coefficients and indices: {}".format(
-                        len(coefficients)))
+                    if self.verbose:
+                        logging.info("Coefficients and indices: {}".format(
+                            len(coefficients)))
+
+                    if self.verbose:
+                        logging.info(
+                            "Adding importances of shape {} for learner {} with score {}"
+                            .format(coefficients.shape, enx, score))
+
+                    self.feature_importances.append((coefficients, feature_names))
                     
-                if self.verbose:
-                    logging.info(
-                        "Adding importances of shape {} for learner {} with score {}"
-                        .format(coefficients.shape, enx, score))
-
-                self.feature_importances.append((coefficients, feature_names))
+                    ## Update the final importance space.
+                    if self.task == "classification":
+                        self.update_global_feature_importances()
+                    
+                except:
+                    logging.info("The considered classifier cannot produce feature importances.")
+                    pass
 
                 single_learner = (learner, individual, score)
                 self.ensemble_of_learners.append(single_learner)
 
-            ## Update the final importance space.
-            if self.task == "classification":
-                self.update_global_feature_importances()
             
         return self
