@@ -10,10 +10,12 @@ import operator
 import pandas as pd
 from collections import defaultdict
 import numpy as np
+from scipy import sparse
 import tqdm
 import multiprocessing as mp
 import scipy.sparse as sps
-
+from sklearn.preprocessing import LabelEncoder
+from scipy.stats.stats import pearsonr
 
 class relationExtractor:
     """
@@ -21,19 +23,25 @@ class relationExtractor:
     """
     def __init__(
         self,
-        max_features=10000,
-        split_char="|||",
-        witem_separator="&&&&",
-        num_cpu=1,
-        neighborhood_token=64,  ## Context window size for relation mapping of words (added in >.34)
-        min_token="bigrams"):
+            max_features=10000,
+            split_char="|||",
+            witem_separator="&&&&",
+            num_cpu=8,
+            neighborhood_token=64,  ## Context window size for relation mapping of words (added in >.34)
+            min_token="bigrams",
+            targets = None,
+            verbose = True):
 
         self.max_features = max_features
         self.neighborhood_token = neighborhood_token
+        self.num_cpu = num_cpu
         self.vocabulary = {}
+        self.verbose = verbose
         self.split_char = split_char
         self.min_token = min_token
+        self.targets = targets
         self.witem_separator = witem_separator
+        self.ranking_conducted = False
 
         if num_cpu == "all":
             self.num_cpu = mp.cpu_count()
@@ -64,43 +72,53 @@ class relationExtractor:
         """
 
         global_distances = {}
+        
         if self.split_char in instance:
             instances = instance.split(self.split_char)
+            
         else:
             instances = [instance]
+            
         for instance in instances:
+            
             if self.min_token == "word":
                 tokens = [x for x in instance.strip().split()]
+                
             elif self.min_token == "bigrams":
                 tokens = []
                 sstring = instance.strip()
                 for enx in range(len(sstring) - 1):
-                    tokens.append("".join([sstring[enx], sstring[enx + 1]]))
+                    tokens.append(sstring[enx:(enx+2)].lower())
+                    
             elif self.min_token == "unigrams":
                 tokens = []
                 sstring = instance.strip()
                 for enx in range(len(sstring) - 1):
                     tokens.append(sstring[enx])
+                    
             token_dict = {token: enx for enx, token in enumerate(tokens)}
-            pairs = combinations(set(tokens), 2)
+            pairs = list(combinations(set(tokens), 2))
+            
             for pair in pairs:
                 w1, w2, distance = self.compute_distance(pair, token_dict)
                 if self.min_token == "word":
                     if distance > self.neighborhood_token:
                         continue
+                    
                 if distance > 2:
                     context_size = int(np.log2(distance))
                     encoded_witem = w1 + "--" + str(context_size) + "--" + w2
                     if not encoded_witem in global_distances:
                         global_distances[encoded_witem] = 0
                     global_distances[encoded_witem] += 1
+                    
         return global_distances
 
     def fit(self, text_vector, b=None):
         """
         Fit the model to a text vector.
 
-        :param text_vector: The input listr of texts.
+        :param text_vector: The input list of texts.
         """
 
         logging.info("Extracting global {} relations".format(self.min_token))
@@ -115,10 +133,37 @@ class relationExtractor:
             v = np.mean(v) # average distance
 
         ## This is the key part -> select the top k features within the context
-        self.wit_vec = sorted(self.global_distances.items(),
-                              key=operator.itemgetter(1),
-                              reverse=True)[0:self.max_features]
 
+        if self.targets is None:
+            self.wit_vec = sorted(self.global_distances.items(),
+                                  key=operator.itemgetter(1),
+                                  reverse=True)[0:self.max_features]            
+        else:
+            self.wit_vec = sorted(self.global_distances.items(),
+                                  key=operator.itemgetter(1),
+                                  reverse=True)[0:(self.max_features*3)]  
+
+            feature_matrix_whole = self.transform(text_vector, custom_shape = len(self.wit_vec))
+            if self.verbose:
+                logging.info(f"Feature ranking of {len(self.wit_vec)} token pairs in progress.")
+                
+            encoder = LabelEncoder()
+            self.targets = encoder.fit_transform(self.targets)
+            correlations = np.zeros(len(self.wit_vec))
+            for el in tqdm.tqdm(range(len(self.wit_vec)), total = len(self.wit_vec)):
+                vec1 = feature_matrix_whole[:,el].todense().A1
+                assert len(self.targets) == len(vec1)
+                correlation = pearsonr(vec1, self.targets)[0]
+                correlations[el] = correlation
+
+            if self.verbose:
+                most_correlated = np.sort(correlations)[::-1][0:10]
+                logging.info("Highest correlations: {most_correlated}")
+                
+            sorted_correlations = set(np.argsort(correlations)[::-1][0:self.max_features].tolist())            
+            self.wit_vec = [x for enx, x in enumerate(self.wit_vec) if enx in sorted_correlations]
+
+            
     def get_feature_names(self):
         """
         Return exact feature names.
@@ -126,13 +171,16 @@ class relationExtractor:
 
         return [x[0] for x in self.wit_vec]
 
-    def transform(self, text_vector):
+    def transform(self, text_vector, custom_shape = None):
         """
         Transform the data into suitable form.
 
         :param text_vector: The input list of texts.
         """
 
+        if custom_shape is None:
+            custom_shape = self.max_features
+
         rs = []
         cs = []
         data = []
@@ -150,7 +198,7 @@ class relationExtractor:
 
         assert len(rs) == len(cs)
         m = sps.csr_matrix((data, (rs, cs)),
-                           shape=(len(text_vector), self.max_features))
+                           shape=(len(text_vector), custom_shape))
         return m
 
     def fit_transform(self, text_vector, a2):
@@ -165,9 +213,11 @@ class relationExtractor:
 
 
 if __name__ == "__main__":
-
-    example_text = pd.read_csv("../data/spanish/train.tsv", sep="\t")['tweet']
-    rex = relationExtractor(min_token="word")
+    
+    tfile = pd.read_csv("../data/spanish/train.tsv", sep="\t")
+    example_text = tfile['tweet']
+    targets = tfile['offensive']
+    rex = relationExtractor(min_token="bigrams")
     rex.fit(example_text)
     m = rex.transform(example_text)
     print(np.count_nonzero(m.todense()) / (5625 * 10000))
